<?php namespace Order\Lib;
use Dever;
class Refund
{
    # 检测是否可以退款
    public function check($id, $name, $data)
    {
        $table = str_replace('_refund', '', $name);
        $order_id = Dever::param('order_id', $data);
        $order_goods_id = Dever::param('order_goods_id', $data);
        $cash = Dever::param('cash', $data);
        $num = Dever::param('num', $data);
        $total_num = Dever::param('total_num', $data);
        $total_cash = Dever::param('total_cash', $data);
        $order_goods = Dever::db($table . '_goods')->find($order_goods_id);
        if (!$order_goods) {
            Dever::alert('申请退款失败');
        }
        if ($total_cash && $cash > $total_cash) {
            Dever::alert('退款金额不能大于剩余金额');
        }
        if ($total_num && $num > $total_num) {
            Dever::alert('退款数量不能大于剩余数量');
        }
    }

    # 发起退款
    public function up_commit($table, $refund)
    {
        $id = Dever::db($table)->insert($refund);
        return $id;
    }

    # 申请退款
    public function apply_commit($id, $name, $data)
    {
        $table = str_replace('_refund', '', $name);
        $info = Dever::db($table . '_refund')->find($id);
        $desc = Dever::param('desc', $data);
        $num = Dever::param('num', $data);
        $cash = Dever::param('cash', $data);
        $status = Dever::param('status', $data);
        $set_type = Dever::param('set_type', $data);
        if ($info && $status == 1 && $info['order_goods_id']) {
            $order_goods = Dever::db($table . '_goods')->find($info['order_goods_id']);
            if ($set_type == 1) {
                if ((!$num || $num <= 0)) {
                    $num = $info['num'] = $update['num'] = $order_goods['num'];
                }
                $info['cash'] = $update['cash'] = $num * $order_goods['cash'];
            }
            if ($set_type == 2) {
                if ((!$cash || $cash <= 0)) {
                    $cash = $info['cash'] = $update['cash'] = $order_goods['cash'];
                }
                $info['num'] = $update['num'] = Dever::number($cash/$order_goods['cash']);
                
            }
            $update['where_id'] = $id;
            Dever::db($table . '_refund')->update($update);
        	$this->log($table, $info, $info['order_id'], $info['order_goods_id'], 2, '申请退款', $desc);
        }
    }

    # 审核退款
    public function audit_commit($id, $name, $data)
    {
        $table = str_replace('_refund', '', $name);
        $audit = Dever::param('audit', $data);
        $audit_text = Dever::param('audit_text', $data);
        $info = Dever::db($table . '_refund')->find($id);
        if ($info && $audit && $info['order_goods_id']) {
            $update['where_id'] = $id;
            $update['status'] = $audit;
            if ($audit == 2) {
                $info['valid'] = $update['valid'] = 1;
                if (!$audit_text) {
                    $audit_text = '同意退款';
                }
                if ($info['type'] > 1 || $info['status'] == 3) {
                    # 可以退款了
                    $update['status'] = 5;
                }
            } elseif ($audit == 3) {
                $update['status'] = 4;
                $info['valid'] = $update['valid'] = 2;
                if (!$audit_text) {
                    $audit_text = '驳回退款';
                }
            }

            Dever::db($table . '_refund')->update($update);

            $refund = Dever::db($table . '_refund')->find(array('order_id' => $info['order_id'], 'status' => 1));
            if ($refund) {
                $status = 2;
            } else {
                if ($info['status'] == 3) {
                    $status = 5;
                } elseif ($info['type'] == 1) {
                    $status = 3;
                } else {
                    $status = 5;
                }
            }
            $info['status'] = $update['status'];

            $this->log($table, $info, $info['order_id'], $info['order_goods_id'], $status, '审核退款', $audit_text);
        }
    }

    private function log($table, $refund, $order_id, $order_goods_id, $status, $info = '', $desc = '', $delivery = array())
    {
        $order_goods = Dever::db($table . '_goods')->find($order_goods_id);
        $goods_info = Dever::load('goods/lib/info.getBaseInfo', $order_goods['goods_id'], $order_goods['sku_id']);
        $info = '[' . $goods_info['aname'] . ']' . $info . ($desc ? ':' . $desc : '');
        Dever::load('order/lib/log')->add($table, $order_id, $info);

        if (isset($order_goods['sell_order_goods_id']) && $order_goods['sell_order_goods_id']) {
            $order = Dever::db($table)->find($order_id);
            $seller = Dever::db('seller/info')->find($order['seller_id']);
            if ($seller && $seller['sell_refund'] == 2) {
                $sell_refund = Dever::db('seller/sell_order_refund')->find(array('order_goods_id' => $order_goods['sell_order_goods_id']));
                if ($sell_refund) {
                    $sell_update['where_id'] = $sell_refund['id'];
                    $sell_update['status'] = $refund['status'];
                    $sell_update['valid'] = $refund['valid'];
                    Dever::db('seller/sell_order_refund')->update($sell_update);
                    Dever::load('order/lib/log')->add('seller/sell_order', $sell_refund['order_id'], $info);
                }
            }
        }
        # 是否同时退采购单
        if (isset($order_goods['buy_order_goods_id']) && $order_goods['buy_order_goods_id']) {
            $order = Dever::db($table)->find($order_id);
            $seller = Dever::db('seller/info')->find($order['seller_id']);
            if ($seller && $seller['sell_refund'] == 2) {
                $buy_order_goods = Dever::db('seller/buy_order_goods')->one($order_goods['buy_order_goods_id']);
                if ($buy_order_goods) {
                    $order_goods['out_type'] = $buy_order_goods['out_type'];
                    $order_goods['out_order_goods_id'] = $buy_order_goods['out_order_goods_id'];

                    Dever::load('order/lib/log')->add('seller/buy_order', $buy_order_goods['order_id'], $info);
                }
            }
        }
        
        if (isset($order_goods['out_type']) && $order_goods['out_type']) {
            $out_table = Dever::load('order/lib/common')->getTable($order_goods['out_type'], '/out_order');
            $out_goods_order = Dever::db($out_table . '_goods')->one($order_goods['out_order_goods_id']);
            Dever::db($out_table)->update(array('where_id' => $out_goods_order['order_id'], 'refund_status' => $status));
            Dever::load('order/lib/log')->add($out_table, $order_goods['order_id'], $info);
        }
    }

    # 同步退货信息到采购单
    public function sync_commit($id, $name, $data)
    {
        $order_num = Dever::param('order_num', $data);
        if ($order_num) {
            $table = str_replace('_refund_delivery', '', $name);
            $delivery = Dever::db($table . '_refund_delivery')->one($id);
            if ($delivery) {
                $refund = Dever::db($table . '_refund')->one($delivery['order_refund_id']);
                $refund['status'] = $update['status'] = 3;
                $update['where_id'] = $refund['id'];
                Dever::db($table . '_refund')->update($update);
                $this->log($table, $refund, $refund['order_id'], $refund['order_goods_id'], 4, '上传退货单号：' . $delivery['order_num'], '', $delivery);
            }
        }
    }

    # 计划任务，将确定退款的订单进行实际退款
    public function cron_commit_api()
    {
        $this->sell();
        $this->buy();
        return 'ok';
    }

    private function buy()
    {
        $table = 'seller/buy_order';
        $table_goods = $table . '_goods';
        $table_refund = $table . '_refund';
        $refund = Dever::db($table_refund)->select(array('status' => 5));

        if ($refund) {
            $log = array();
            $refund_order = array();
            foreach ($refund as $k => $v) {
                Dever::db($table_refund)->update(array('where_id' => $v['id'], 'status' => 6));
                # 退款代码 开始
                $status = 7;
                if ($status == 7 && $v['order_goods_id']) {
                    $this->buyOne($table, $table_goods, $v, $log);
                }
                # 退款代码 结束
                Dever::db($table_refund)->update(array('where_id' => $v['id'], 'status' => $status));
                $refund_order[$v['order_id']] = $v['order_id'];
            }

            if ($log) {
                foreach ($log as $k => $v) {
                    foreach ($v as $k1 => $v1) {
                        $info = '[' . implode('、', $v1['goods']) . '] 退货数量：' . $v1['num'];
                        if (isset($v1['cash'])) {
                            $info .= ' 退款金额：' . $v1['cash'];
                        }
                        
                        Dever::load('order/lib/log')->add($k, $k1, $info);
                    }
                }
            }

            # 将整个订单退款
            foreach ($refund_order as $k => $v) {
                $order = Dever::db($table)->find($v);
                if ($order['status'] <= 5) {
                    $order_refund = Dever::db($table_refund)->getTotal(array('order_id' => $v, 'status' => 7));
                    if ($order['num'] <= $order_refund['num']) {
                        $update = array();
                        $update['where_id'] = $v;
                        $update['status'] = 9;
                        $update['finish_date'] = time();
                        Dever::db($table)->update($update);
                    }
                }
            }
        }
    }

    private function buyOne($table, $table_goods, $v, &$log)
    {
        $order_goods = Dever::db($table_goods)->find($v['order_goods_id']);
        $goods_info = Dever::load('goods/lib/info.getBaseInfo', $order_goods['goods_id'], $order_goods['sku_id'], $order_goods['unit_id']);

        if ($goods_info) {
            if ($v['num'] > $order_goods['num']) {
                $v['num'] = $order_goods['num'];
            }
            if ($v['cash'] <= 0) {
                $v['cash'] = $order_goods['cash'] * $v['num'];
            }
            $tui = $this->account($table, $v, $v['cash'], 'caigou', '采购单');
            if ($tui <= 0) {
                $status = 5;
            } else {
                $order_id = $order_goods['order_id'];
                if (!isset($log[$table][$order_id])) {
                    $log[$table][$order_id] = array();
                    $log[$table][$order_id]['goods'] = array();
                    $log[$table][$order_id]['num'] = 0;
                    $log[$table][$order_id]['cash'] = 0;
                }
                $key = $order_goods['goods_id'] .'_'. $order_goods['sku_id'] .'_'. $order_goods['unit_id'];
                $log[$table][$order_id]['goods'][$key] = $goods_info['aname'];
                $log[$table][$order_id]['num'] += $v['num'];
                $log[$table][$order_id]['cash'] += $v['cash'];

                # 处理出库单
                if (isset($order_goods['out_type']) && $order_goods['out_type']) {
                    $out_table = Dever::load('order/lib/common')->getTable($order_goods['out_type'], '/out_order');
                    $out_goods_order = Dever::db($out_table . '_goods')->one($order_goods['out_order_goods_id']);
                    Dever::db($out_table)->update(array('where_id' => $out_goods_order['order_id'], 'refund_status' => 6));

                    $order_id = $out_goods_order['order_id'];
                    if (!isset($log[$out_table][$order_id])) {
                        $log[$out_table][$order_id] = array();
                        $log[$out_table][$order_id]['goods'] = array();
                        $log[$out_table][$order_id]['num'] = 0;
                    }
                    $log[$out_table][$order_id]['goods'][$key] = $goods_info['aname'];
                    $log[$out_table][$order_id]['num'] += $v['num'];
                }
            }
        }
    }

    private function sell()
    {
        $table = 'seller/sell_order';
        $table_goods = $table . '_goods';
        $table_refund = $table . '_refund';
        $refund = Dever::db($table_refund)->select(array('status' => 5));
        if ($refund) {
            $log = array();
            $refund_order = array();
            foreach ($refund as $k => $v) {
                Dever::db($table_refund)->update(array('where_id' => $v['id'], 'status' => 6));
                # 退款代码 开始
                $status = 7;
                if ($status == 7 && $v['order_goods_id']) {
                    $this->sellOne($table, $table_goods, $v, $log);
                }
                # 退款代码 结束
                
                Dever::db($table_refund)->update(array('where_id' => $v['id'], 'status' => $status));
                $refund_order[$v['order_id']] = $v['order_id'];
            }

            if ($log) {
                foreach ($log as $k => $v) {
                    foreach ($v as $k1 => $v1) {
                        $info = '[' . implode('、', $v1['goods']) . '] 退货数量：' . $v1['num'];
                        if (isset($v1['cash'])) {
                            $info .= ' 退款金额：' . $v1['cash'];
                        }
                        
                        Dever::load('order/lib/log')->add($k, $k1, $info);
                    }
                }
            }

            # 将整个订单退款
            foreach ($refund_order as $k => $v) {
                $order = Dever::db($table)->find($v);
                if ($order['status'] <= 5) {
                    $order_refund = Dever::db($table_refund)->getTotal(array('order_id' => $v, 'status' => 7));
                    if ($order['num'] <= $order_refund['num']) {
                        $update = array();
                        $update['where_id'] = $v;
                        $update['status'] = 9;
                        $update['finish_date'] = time();
                        Dever::db($table)->update($update);
                        if (isset($order['buy_order_id']) && $order['buy_order_id']) {
                            $seller = Dever::db('seller/info')->find($order['seller_id']);
                            $buy_order = Dever::db('seller/buy_order')->find($order['buy_order_id']);
                            if ($seller && $seller['sell_refund'] == 2 && $buy_order && $buy_order['status'] <= 5) {
                                $update['where_id'] = $buy_order['id'];
                                Dever::db('seller/buy_order')->update($update);
                            }
                        }
                    }
                }
            }
        }
    }

    private function sellOne($table, $table_goods, $v, &$log)
    {
        $order_goods = Dever::db($table_goods)->find($v['order_goods_id']);
        $goods_info = Dever::load('goods/lib/info.getBaseInfo', $order_goods['goods_id'], $order_goods['sku_id'], $order_goods['unit_id']);

        if ($goods_info) {
            if ($v['num'] > $order_goods['num']) {
                $v['num'] = $order_goods['num'];
            }
            if ($v['cash'] <= 0) {
                $v['cash'] = $order_goods['cash'] * $v['num'];
            }
            $tui = $this->account($table, $v, $v['cash'], 'goumai', '订单');
            $tui = $this->account($table, $v, $v['cash'], 'xiaoshou', '销售单');

            if ($tui <= 0) {
                $status = 5;
            } else {
                $order_id = $order_goods['order_id'];
                if (!isset($log[$table][$order_id])) {
                    $log[$table][$order_id] = array();
                    $log[$table][$order_id]['goods'] = array();
                    $log[$table][$order_id]['num'] = 0;
                    $log[$table][$order_id]['cash'] = 0;
                }
                $key = $order_goods['goods_id'] .'_'. $order_goods['sku_id'] .'_'. $order_goods['unit_id'];
                $log[$table][$order_id]['goods'][$key] = $goods_info['aname'];
                $log[$table][$order_id]['num'] += $v['num'];
                $log[$table][$order_id]['cash'] += $v['cash'];

                if (isset($order_goods['buy_order_goods_id']) && $order_goods['buy_order_goods_id']) {
                    $order = Dever::db($table)->find($order_id);
                    $seller = Dever::db('seller/info')->find($order['seller_id']);
                    if ($seller && $seller['sell_refund'] == 2) {
                        $v['order_goods_id'] = $order_goods['buy_order_goods_id'];
                        $v['cash'] = 0;
                        $this->buyOne('seller/buy_order', 'seller/buy_order_goods', $v, $log);
                    }
                }
            }
        }
    }

    # 处理退款
    public function account($table, $order, $cash, $name, $desc)
    {
        if (isset($order['order_id'])) {
            $order = Dever::db($table)->find($order['order_id']);
        }
        
        $seller = Dever::db('seller/info')->find($order['seller_id']);
        if ($name == 'goumai') {
            $uid = $order['uid'];
        } else {
            $uid = $seller['uid'];
        }
        $state = Dever::load('account/lib/info.up_commit', $uid, $cash, $name . '_tuikuan', 'qianbao', $desc . '退款:'. $order['order_num'], $table, $order['id'], 1, false);

        return $state;
    }
}
