<?php

namespace Goods\Lib;

use Dever;

class Info
{
    # 获取商品列表
    public function getList()
    {
    	
    }

    # 获取商品详情
    public function getInfo($id, $type = 1)
    {
        # type == 1 是销售商品 2是采购商品
        $info = is_array($id) ? $id : Dever::db('goods/info')->getOne($id);

        if (!$info) {
            Dever::alert('商品信息不存在');
        }

        $info = $this->get($info, $type);

        return $info;
    }

    private function get($info, $type = 1)
    {
        # 基本信息
        $info['udate'] = date('Y-m-d H:i', $info['udate']);
        $info['cdate'] = date('Y-m-d H:i', $info['cdate']);

        # 获取分类
        $info['category_info'] = Dever::load('goods/lib/category')->getInfo($info['category']);

        # 获取属性
        $attr = Dever::db('goods/info_attr')->select(array('info_id' => $info['id']));
        if ($attr) {
            foreach ($attr as $k => $v) {
                $attr_info = Dever::load('goods/lib/attr')->getInfo($v['attr_id'], $v['attr_value']);
                if ($attr_info) {
                    $info['attr'][] = $attr_info;
                }
            }
        }

        # 获取规格
        $info['spec'] = Dever::load('goods/lib/spec')->getData($info['id'], 'getData');

        # 获取价格
        $info['sku'] = Dever::load('goods/lib/sku')->getData($info['id'], false, $type);

        # 获取组合商品
        if ($info['type'] > 10) {
            $info['goods'] = Dever::json_decode($info['goods']);
        } else {
            unset($info['goods']);
        }

        # 获取单位信息
        if ($info['unit_id'] > 0) {
            $info['unit'] = Dever::db('goods/unit')->find($info['unit_id']);
        }

        return $info;
    }

    # 获取商品及其属性列表
    public function getSetList($table, $other_where, $cate_id = false, $col = 'goods_id', $set = array(), $price_id = -1, $source = '')
    {
        $price = Dever::input('price', 0);
        $buy_price = Dever::input('buy_price', 0);
        $total = Dever::input('total', 1);
        if ($cate_id) {
            $where['top_category_id'] = $cate_id;
        }
        $where['status'] = 1;

        if (!$source) {
            $source = 'goods/info';
        }
        $data = Dever::db($source)->getData($where);

        $result = array();
        if ($data) {
            $i = 0;
            foreach ($data as $k => $v) {
                
                $sku = Dever::db('goods/info_sku')->select(array('info_id' => $v['id']));

                if ($sku) {
                    $other = array();
                    if ($table) {
                        $other_where[$col] = $v['id'];
                        $other_where['sku_id'] = $sku[0]['id'];
                        $other = Dever::db($table)->find($other_where);
                    }
                    
                    if ($set) {
                        $result[$i]['set'] = $set;
                    }
                    
                    $result[$i]['id'] = $v['id'];
                    $result[$i]['name'] = $v['name'];
                    $result[$i]['type'] = $v['type'];
                    $result[$i]['stock'] = $v['stock'];
                    $result[$i]['commission'] = $v['commission'];
                    $result[$i]['top_category_id'] = $v['top_category_id'];
                    $result[$i]['sell_type'] = $v['sell_type'];
                    $result[$i]['buy_type'] = $v['buy_type'];

                    $result[$i]['old_price'] = $result[$i]['price'] = $price ? $price : $sku[0]['price'];
                    $result[$i]['old_buy_price'] = $result[$i]['buy_price'] = $buy_price ? $buy_price : $sku[0]['buy_price'];
                    $result[$i]['old_cost_price'] = $result[$i]['cost_price'] = $sku[0]['cost_price'];
                    if (strstr($table, 'supplier')) {
                        $result[$i]['cost_price'] = '-';
                    }
                    $result[$i]['select'] = 2;
                    $result[$i]['del'] = 1;
                    $result[$i]['total'] = isset($v['total']) ? $v['total'] : 0;

                    if ($price_id > 0 && $sku[0]['key'] == -1) {
                        list($result[$i]['price'], $result[$i]['buy_price']) = Dever::load('goods/lib/price')->get($price_id, $v['id'], $sku[0]['id'], $result[$i]['price'], $result[$i]['buy_price']);
                    }
                    
                    if ($other) {
                        if (isset($other['commission']) && $other['commission']) {
                            $result[$i]['commission'] = $other['commission'];
                        }
                        if (isset($other['price']) && $other['price']) {
                            $result[$i]['price'] = $other['price'];
                        }
                        if (isset($other['buy_price']) && $other['buy_price']) {
                            $result[$i]['buy_price'] = $other['buy_price'];
                        }
                        if (isset($other['cost_price'])) {
                            $result[$i]['cost_price'] = $other['cost_price'];
                        }
                        if (isset($other['sell_min']) && $other['sell_min']) {
                            $result[$i]['sell_min'] = $other['sell_min'];
                        }
                        if (isset($other['buy_min']) && $other['buy_min']) {
                            $result[$i]['buy_min'] = $other['buy_min'];
                        }
                        if (isset($other['source_id']) && $other['source_id']) {
                            $result[$i]['source_id'] = $other['source_id'];
                        }
                        if (isset($other['in_num'])) {
                            if (isset($other['out_num'])) {
                                $result[$i]['total'] = $other['in_num'] - $other['out_num'];
                            } else {
                                $result[$i]['total'] = $other['in_num'];
                            }
                            
                        } elseif (isset($other['num'])) {
                            $result[$i]['total'] = $other['num'];
                        }
                        
                        $result[$i]['select'] = $other['state'];
                    }

                    $result[$i]['total'] = $result[$i]['total'] * $total;
                    $result[$i]['children'] = array();
                    $num = count($sku);
                    if ($sku[0]['key'] == -1) {
                        $result[$i]['id'] .= '-' . $sku[0]['id'];
                        $result[$i]['old_price'] = $sku[0]['price'];
                        $result[$i]['old_buy_price'] = $sku[0]['buy_price'];

                        if (!$price && $result[$i]['price'] <= 0) {
                            $result[$i]['price'] = $sku[0]['price'];
                        }
                        if (!$buy_price && $result[$i]['buy_price'] <= 0) {
                            $result[$i]['buy_price'] = $sku[0]['buy_price'];
                        }
                        
                        //$result[$i]['id'] .=  '-' . $sku[0]['id'];
                        $result[$i]['end'] = true;
                    } else {
                        foreach ($sku as $k1 => $v1) {
                            $v1['sku_name'] = '';
                            if ($v1['key']) {

                                $key = str_replace('-', ',', $v1['key']);
                                $spec = Dever::db('goods/info_spec_value')->getGroupData(array('ids' => $key));

                                $v1['sku_name'] = $spec['name'];
                            }

                            $name = $v['name'];
                            if ($v1['sku_name']) {
                                $name .= '-' . $v1['sku_name'];
                            }

                            $children = array
                            (
                                'id' => $v['id'] . '-' . $v1['id'],
                                'name' => $name,
                                'stock' => $v['stock'],
                                'commission' => $v['commission'],
                                'sell_type' => $v['sell_type'],
                                'buy_type' => $v['buy_type'],
                                'top_category_id' => $v['top_category_id'],
                                'old_price' => $v1['price'],
                                'old_buy_price' => $v1['buy_price'],
                                'old_cost_price' => $v1['cost_price'],
                                'price' => $price ? $price : $v1['price'],
                                'buy_price' => $buy_price ? $buy_price : $v1['buy_price'],
                                'cost_price' => $v1['cost_price'],
                                'type' => $v['type'],
                                'del' => 1,
                                'end' => true,
                                'total' => 0,
                            );
                            if (strstr($table, 'supplier')) {
                                $children['cost_price'] = '-';
                            }
                            $other_sku = array();
                            if ($table) {
                                $other_where['sku_id'] = $v1['id'];
                                $other_sku = Dever::db($table)->find($other_where);
                            }

                            $children['select'] = 2;
                            $children['total'] = 0;
                            if ($set) {
                                $children['set'] = $set;
                            }
                            if ($price_id > 0) {
                                list($children['price'], $children['buy_price']) = Dever::load('goods/lib/price')->get($price_id, $v['id'], $v1['id'], $children['price'], $children['buy_price']);
                            }
                            if ($other_sku) {
                                if (isset($other_sku['commission']) && $other_sku['commission']) {
                                    $children['commission'] = $other_sku['commission'];
                                }
                                if (isset($other_sku['price']) && $other_sku['price']) {
                                    $children['price'] = $other_sku['price'];
                                }
                                if (isset($other_sku['buy_price']) && $other_sku['buy_price']) {
                                    $children['buy_price'] = $other_sku['buy_price'];
                                }
                                if (isset($other_sku['cost_price'])) {
                                    $children['cost_price'] = $other_sku['cost_price'];
                                }
                                if (isset($other_sku['sell_min']) && $other_sku['sell_min']) {
                                    $children['sell_min'] = $other_sku['sell_min'];
                                }
                                if (isset($other_sku['buy_min']) && $other_sku['buy_min']) {
                                    $children['buy_min'] = $other_sku['buy_min'];
                                }
                                if (isset($other_sku['source_id']) && $other_sku['source_id']) {
                                    $children['source_id'] = $other_sku['source_id'];
                                }
                                if (isset($other_sku['in_num'])) {
                                    if (isset($other_sku['out_num'])) {
                                        $children['total'] = $other_sku['in_num'] - $other_sku['out_num'];
                                    } else {
                                        $children['total'] = $other_sku['in_num'];
                                    }
                                } elseif (isset($other_sku['num'])) {
                                    $children['total'] = $other_sku['num'];
                                }
                                
                                $children['select'] = $other_sku['state'];
                            }

                            $children['total'] = $children['total']*$total;
                            $result[$i]['children'][] = $children;
                        }
                    }
                    $i++;
                }
            }
        }

        return $result;
    }

    # 获取商品的基本信息，只包括名称-价格等信息
    public function getBaseInfo($info, $sku_id = false, $unit_id = false, $type = 'buy')
    {
        $info = is_array($info) ? $info : Dever::db('goods/info')->one($info);

        $info['base_unit_id'] = $info['unit_id'];
        if ($unit_id) {
            $info['unit_id'] = $unit_id;
        }

        $info['unit'] = '';
        if ($info['unit_id']) {
            $unit = Dever::db('goods/unit')->find($info['unit_id']);
            if ($unit) {
                $info['unit'] = $unit['name'];
            }
        }

        if (!$info['unit']) {
            $info['unit'] = '个';
        }
        $info['price'] = $info['buy_price'] = $info['cost_price'] = 0;
        $info['code'] = '';
        $info['aname'] = $info['name'];
        $info['spec_name'] = array();

        
        if ($sku_id && $sku_id > 0) {
            $where = array('info_id' => $info['id']);
            $where['id'] = $sku_id;

            $sku = Dever::db('goods/info_sku')->find($where);
            if ($sku) {
                $info['code'] = $sku['code'];
                $info['price'] = $sku['price'];
                $info['buy_price'] = $sku['buy_price'];
                $info['cost_price'] = $sku['cost_price'];
                if ($sku['key'] != -1) {
                    $key = str_replace('-', ',', $sku['key']);
                    $spec = Dever::db('goods/info_spec_value')->getGroupData(array('ids' => $key));
                    if ($spec) {
                        $info['aname'] .= '-' . $spec['name'];
                        $info['spec_name'][] = $spec['name'];
                    }
                }
            } else {
                return false;
            }
        }

        if (!$info['price']) {
            return false;
        }

        if (!$info['spec_name']) {
            $info['spec_name'] = '无';
        } else {
            $info['spec_name'] = implode(',', $info['spec_name']);
        }

        $pic = explode(',', $info['pic']);
        $info['cover'] = $pic[0];

        $this->getGroup($info, $type);

        return $info;
    }

    # 获取组合商品
    public function getGroup(&$info, $type)
    {
        $info['goods'] = array();
        if ($info['type'] == 11 || $info['type'] == 12) {
            $goods = Dever::db('goods/info_goods')->select(array('info_id' => $info['id']));
            $info['goods'] = array();
            foreach ($goods as $k => $v) {
                $goods_info = $this->getBaseInfo($v['goods_id'], $v['sku_id']);
                if ($goods_info) {
                    if ($type == 'buy') {
                        $cash = $goods_info['price'];
                    } elseif ($type == 'sell') {
                        $cash = $goods_info['buy_price'];
                    } else {
                        $cash = $goods_info['cost_price'];
                    }
                    $info['price'] += $goods_info['price'];
                    $info['buy_price'] += $goods_info['buy_price'];
                    $info['cost_price'] += $goods_info['cost_price'];
                    $info['goods'][] = array
                    (
                        'id' => $goods_info['id'],
                        'code' => $goods_info['code'],
                        'aname' => $goods_info['aname'],
                        'name' => $goods_info['name'],
                        'spec_name' => $goods_info['spec_name'],
                        'cash' => $cash,
                        'price' => $goods_info['price'],
                        'buy_price' => $goods_info['buy_price'],
                        'cost_price' => $goods_info['cost_price'],
                        'cover' => $goods_info['cover'],
                        'unit_id' => $goods_info['unit_id'],
                        'num' => $v['num'],
                        'unit' => $goods_info['unit'],
                    );
                }
            }
        }
    }
}
